import configparser
import json
import requests

_headKey    = "faspay-key"
_headTime   = "faspay-timestamp"
_headSign   = "faspay-signature"
_headAuth   = "faspay-authorization"

_host = dict()
_host["development"]    = "https://dev2.faspay.co.id"
_host["production"]     = "https://web.faspay.co.id"

_path = dict()
_path["get_token"]          = "/account/api/tokens"
_path["register"]           = "/account/api/register"
_path["register_confirm"]   = "/account/api/confirm"
_path["transfer"]           = "/account/api/transfer"
_path["balance_inquiry"]    = "/account/api/balance_inquiry"
_path["inquiry_name"]       = "/account/api/inquiry_name"
_path["mutasi"]             = "/account/api/mutasi"
_path["inquiry_status"]     = "/account/api/inquiry_status"

_method = dict()
_method["get_token"]        = "GET"
_method["register"]         = "POST"
_method["register_confirm"] = "POST"
_method["transfer"]         = "POST"
_method["balance_inquiry"]  = "GET"
_method["inquiry_name"]     = "GET"
_method["mutasi"]           = "GET"
_method["inquiry_status"]   = "GET"

_env                        = "development"
_section                    = "get_token"
_url                        = None
_current_method             = None
_current_path               = None
_content_type               = "application/json"
_headers                    = {}
_reqd                       = dict()
_reqx                       = None
_rspd                       = dict()
_rspx                       = None
_virtual_account            = None
_info                       = {}

def setEnvironment(self, env):
    self._env = env

def getEnvironment():
    return _env

def setSection(self, section):
    self._section = self._info["section"] = section

def getSection():
    return _section

def setRequestParam(self, data={}):
    config = configparser.ConfigParser()
    config.read('SendMeConfig.ini')
    if config.sections() == []: raise Exception("File Config not found. Please create file config based on Readme.md")

    try:
        data["virtual_account"]
    except :
        self.virtual_account = config.get(_env, "virtual_account")
        data.update({"virtual_account" : self.virtual_account})

    self._reqd = data
    #self._reqd = self._info["request"]["array"] = data
    __array2json(self)

def getResponseParam(self):
    __json2array(self)
    return _rspd

def setHeaders(self, valueKey, valueTime, valueSign, valueAuth):
    self._headers = self._info["headers"] = {
        _headKey : valueKey,
        _headTime : valueTime,
        _headSign : valueSign,
        _headAuth : valueAuth
    }

def getMethod(self):
    self._current_method = _method[self.getSection()]
    return _current_method

def getPath(self):
    self._current_path = _path[self.getSection()]
    return _current_path

def generateUrl(self):
    if _env == False or _section == False:
        return False

    host = self._info["host"] = _host[_env]
    path = self._info["path"] = _path[_section]
    self._url = self._info["url"] = host + path
    self._current_method = self._info["method"] = _method[_section]

    if _current_method == "GET" and len(_reqd) > 0:
        arrVal = []
        for x in _reqd: arrVal.append(_reqd[x])
        queryString = "/" + "/".join(arrVal)
        self._url = self._info["url"] = _url + queryString

def __array2json(self, param={}):
    if param == {} : param = _reqd

    if isinstance(param, list):
        self._reqx = json.dumps(param)

    #self._info["request"]["json"] = _reqx

def __json2array(self, param={}):
    if param == {} : param = _rspx

    if isinstance(param, str):
        self._rspd = json.loads(param)

    #self._info["response"]["array"] = _rspd
    #self._info["response"]["json"]  = _rspx

def curl(self):
    result = None
    self._headers["Accept"] = _content_type
    self._headers["Content-Type"] = _content_type

    self._info["headers"] = _headers

    r = requests.post(_url, json=_reqx, headers=_headers)
    result = r.text

    if r.status_code != 200: result = json.dumps({"error":True, "message":r.status_code})

    self._rspx = result


import configparser
import datetime
import base64
import hashlib
from Crypto import Random
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from FaspaySendme import _http

_config         = None;
_faspayKey      = None;
_timestamp      = None;
_signature      = None;
_authorization  = None;
_token          = None;
_accessToken    = None;
_errors         = {};

def generateHeadersRequest(self):
    self._config = configparser.ConfigParser()
    self._config.read('SendMeConfig.ini')
    if _config.sections() == []: raise Exception("File Config not found. Please create file config based on Readme.md")

    self._timestamp = datetime.datetime.today().strftime("%Y-%m-%d %H:%M:%S")
    self._faspayKey = _config.get(_http.getEnvironment(), "faspay_key")
    self._authorization = self.generateAuthorization(self)

    self._accessToken = getToken(self)
    generateToken(self)
    self._signature = generateSignature(self)
    _http.setHeaders(_http, _faspayKey, _timestamp, _signature, _authorization)

def getToken(self):
    section = "get_token"
    generateToken(self, section)
    self._signature = generateSignature(self, _http._method[section])
    _http.setHeaders(_http, _faspayKey, _timestamp, _signature, _authorization)
    _http._url = _http._host[_http.getEnvironment()] + _http._path[section]
    _http.curl(_http)
    rspd = _http.getResponseParam(_http)

    if len(rspd) > 0 and rspd["status"] == "2":
        return rspd["access_token"]
    else:
        print(rspd);exit();

    return False

def generateAuthorization(self):
    faspaySecret    = _config.get(_http.getEnvironment(), "faspay_secret")
    appKey          = _config.get(_http.getEnvironment(), "app_key")
    appSecret       = _config.get(_http.getEnvironment(), "app_secret")

    string = appKey + ":" + appSecret
    auth = self.encryptAES256(string, faspaySecret)
    return auth

def generateToken(self, section=None):
    if section == "get_token":
        string = _config.get(_http.getEnvironment(), "client_key") + ":" + _config.get(_http.getEnvironment(), "client_secret")
        self._token = base64.b64encode(string.encode()).decode()
    else:
        string = _config.get(_http.getEnvironment(), "client_key") + ":" + _config.get(_http.getEnvironment(), "client_secret") + ":" + _accessToken
        self._token = base64.b64encode(string.encode()).decode()

def generateSignature(self, method=None, path=None):
    requestBody = ""

    if method == None: method = _http.getMethod(_http)
    if path == None: path = _http.getPath(_http)

    stringToSign = [method, path, _timestamp, _token, requestBody]
    stringToSign = ":".join(stringToSign)
    sign = self.encryptAES256(stringToSign, _config.get(_http.getEnvironment(), "faspay_secret"))
    return sign

def encryptAES256(string, key):
    plaintext   = string
    method      = AES.MODE_CBC
    iv          = _config.get(_http.getEnvironment(), "iv")

    passwd    = hashlib.sha256(key.encode()).digest()[0:32]
    iv        = hashlib.md5(key.encode() + iv.encode()).hexdigest()[-16:]
    raw       = pad(plaintext.encode(), AES.block_size)

    cipher    = AES.new(passwd, method, iv.encode())
    ciphEnc   = cipher.encrypt(raw)
    enc       = base64.b64encode(ciphEnc)
    return enc.decode()

def setErrors(self, code, message):
    self._errors = {
        "error" : True,
        "code" : code,
        "message" : message
    }


from FaspaySendme import _request, _http

class Services():
    _environment = "development"
    _request = None

    def enableProd(self):
        self._environment = "production"

    def register(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "register")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def confirm(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "register_confirm")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def transfer(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "transfer")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def balance_inquiry(self):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "balance_inquiry")
        _http.setRequestParam(_http)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def inquiry_name(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "inquiry_name")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def mutasi(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "mutasi")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)

    def inquiry_status(self, data={}):
        _http.setEnvironment(_http, self._environment)
        _http.setSection(_http, "inquiry_status")
        _http.setRequestParam(_http, data)
        _request.generateHeadersRequest(_request)
        _http.generateUrl(_http)
        _http.curl(_http)
        return _http.getResponseParam(_http)



from setuptools import setup, find_packages

setup(name='FaspaySendme',
      version='1.1',
      url='https://github.com/faspay-team/faspay-sendme-python',
      license='MIT',
      author='Vincentius Setyo K',
      author_email='vincentius.setyo@faspay.co.id',
      description='This package provides Faspay SendMe 1.1 support for the Python Language.',
      packages=find_packages(exclude=['tests']),
      long_description=open('README.md').read(),
      long_description_content_type="text/markdown",
      zip_safe=False)